//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Interface/CrosscorrelationModels.h
//! @brief     Define CrosscorrelationModel classes.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2024
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_INTERFACE_CROSSCORRELATIONMODELS_H
#define BORNAGAIN_SAMPLE_INTERFACE_CROSSCORRELATIONMODELS_H

#include "Base/Type/ICloneable.h"
#include "Param/Node/INode.h"
#include <heinz/Vectors3D.h>

//! Base class for cross-correlation function models.
class CrosscorrelationModel : public ICloneable, public INode {
public:
    CrosscorrelationModel* clone() const override = 0;
    virtual double crossCorrLength(const R3& k) const = 0;
    virtual double replicationFactor(double thickness) const = 0;

#ifndef SWIG
    //! Used to create the Python constructor of this class
    virtual std::string pythonArguments() const = 0;
#endif
};

//! The crosscorrelation factor depends on the distance between interfaces and does not depend
//! on the spatial frequency of roughness.
class FixedDepthCrosscorrelation : public CrosscorrelationModel {
public:
    FixedDepthCrosscorrelation(double cross_corr_depth = 0);
    FixedDepthCrosscorrelation* clone() const override;
    std::string className() const override { return "FixedDepthCrosscorrelation"; }
    std::string validate() const override;
    double crossCorrLength(const R3&) const override { return m_crosscorrelation_depth; }
    double replicationFactor(double thickness) const override;
    std::vector<ParaMeta> parDefs() const final { return {{"CrossCorrDepth", "nm"}}; }

#ifndef SWIG
    std::string pythonArguments() const override;
#endif

private:
    double m_crosscorrelation_depth = 0;
};

#endif // BORNAGAIN_SAMPLE_INTERFACE_CROSSCORRELATIONMODELS_H
